{% extends "admin_layout.jinja2" %}
{% block body %}
<style>
    body {
        padding: 0;
        margin: 0;
        font-family: sans-serif;
    }

    h1 {
        margin: 0;
        padding: 20px;
        background-color: #3f51b5;
        color: #ffffff;
    }

    .container {
        padding: 20px;
    }

    a, a:link, a:visited{
        color: #1b61d6;
        text-decoration: none;
    }
    a:hover{
        color: #b54926;
    }

</style>

<h1>Channelstream Admin</h1>

<div class="container">

    <p>
        <a href="{{ request.route_url('pyramid_apispec.api_explorer_path') }}">Api Explorer</a>
        <a href="{{ request.route_url('admin_action', action="sign_out") }}"> Sign Out</a>
    </p>

    <channelstream-admin></channelstream-admin>
</div>
{% endblock %}
